import numpy,LP
import ScrumPy
import math,types


from Data import DataSets, Stats, Histo
from Util import Set,Sci

import BuildLP  
from LP import glpk



Photon = "Photon_tx"


from ScrumPy.Data import DataSets # results will be returned in a DataSet so we must import this

import BuildLP # this module is specific to this exercise and can be found in ../Model/Tools



def LightScan(m,lo,hi,nPoints, lp=None):
    '''
        pre : m      -  the model 
              lo      - lower limit for Photon flux 
              hi      - upper limit for Photon flux
              nPoints - number of points to sample
              lp      - an lp object, if None a new one will be generated
                
       post:  returns a dataset (ds) containing the scan results. ds.lp will be the lp object
    '''

    
    rv = DataSets.DataSet() # where our results will be stored
                            # rv stands for "Return Value"

    
    if lp == None:              # if the lp object is equal to None, build a new one
        lp = BuildLP.BiomassLP(m)
    
    Photon = lo                    # lower value for the Photon flux
    incr = (hi-lo)/(nPoints-1.0)  # the step size for the Photon flux from the range and number of points
    
    for n in range(nPoints):                # for each step
        lp.SetFixedFlux({'Photon_tx':Photon})  # set the "Photon_tx" flux in the lp to the current Photon value
        lp.Solve()                              # solve the lp
        if lp.GetStatusMsg() == "Optimal":
                    rv.UpdateFromDic(lp.GetPrimSol())   # update the returned data-set with the current solution
        Photon += incr                      # update the current value of Photon

    rv.MakeFirstCol("Photon_tx")   # make the first column Photon - default X axis for plotting
    rv.lp = lp                  # add the lp to the datset - make it available to the user

    return rv # return the results to the user - don't forget to do this !


def GetChangers(ds, tol=1e-6):
    """ pre: ds = LightScan(...)
       post: GetChangers(ds) => list of reactions in ds whose flux changes by more than tol
    """

    rv = []

    for c in ds.cnames[1:]:  # ignore first col, it's the Photon_tx
        if abs(ds.MaxInCol(c) - ds.MinInCol(c)) > tol:
            rv.append(c)

    return rv

def GetSwitchers(ds):
    """ pre: ds = ATPScan(...)
       post: GetSwitchers(ds) => list of reactions that have at least one value of zero
    """
    
    rv = []

    for c in ds.cnames[1:]:  # ignore first col, it's the Photon_tx
        if min(map(abs, ds.GetCol(c))) == 0.0:
            rv.append(c)

    return rv


def GetRanges(ds, tol=1e-6):
    """ pre: ds = ATPScan(...)
       post: GetRanges(ds) => dictionary mapping reactions -> max - min flux
    """
    rv = {}

    for c in ds.cnames[1:]:
        fluxrange = abs( ds.MaxInCol(c) - ds.MinInCol(c))
        if fluxrange > tol:
            rv[c] = fluxrange

    return rv
    

    

        
